@@ -14,7 +14,7 @@ use App::Git::Workflow;
 use App::Git::Workflow::Command qw/get_options/;
 use utf8;
 
-our $VERSION  = 0.96009;
+our $VERSION  = 0.96011;
 our $workflow = App::Git::Workflow->new;
 our ($name)   = $PROGRAM_NAME =~ m{^.*/(.*?)$}mxs;
 our %option;
@@ -30,17 +30,20 @@ sub run {
         'all|a',
         'fmt|format|f=s',
         'changes|c',
+        'commits|C',
         'min|min-commits|M=i',
-        'date|d=s',
+        'since|s=s',
+        'until|u=s',
         'period|p=s',
+        'periods|P=i',
         'merges|m!',
     );
 
     my %users;
     my $commits = 0;
-    my $date = $option{date};
+    my $since = $option{since};
 
-    if (!$date) {
+    if (!$since) {
         my $now = localtime;
         my $period
             = $option{period} eq 'day'   ? 1
@@ -48,7 +51,7 @@ sub run {
             : $option{period} eq 'month' ? 30
             : $option{period} eq 'year'  ? 365
             :                              die "Unknown period '$option{period}' please choose one of day, week, month or year\n";
-        $date
+        $since
             = $now->wday == 1 ? localtime(time - 3 * $period * 24 * 60 * 60)->ymd
             : $now->wday == 7 ? localtime(time - 2 * $period * 24 * 60 * 60)->ymd
             :                   localtime(time - 1 * $period * 24 * 60 * 60)->ymd;
@@ -57,22 +60,41 @@ sub run {
     my @options;
     push @options, '-r' if $option{remote};
     push @options, '-a' if $option{all};
+    my @log = (
+        '--format=format:%h %an',
+        ($option{merges} ? () : '--no-merges'),
+    );
 
-    for my $branch ($workflow->git->branch(@options)) {
-        next if $branch =~ / -> /;
-        $branch =~ s/^[*]?\s*//;
-        for my $log ($workflow->git->log('--format=format:%h %an', ($option{merges} ? () : '--no-merges'), "--since=$date", $branch)) {
-            my ($hash, $name) = split /\s/, $log, 2;
-            $users{$name}{$hash} = 1;
-            $commits++;
+    my $periods = $option{periods} || 1;
+    while ($periods--) {
+        my @dates;
+        if ($option{periods}) {
+            @dates = $self->dates($option{period}, $option{periods}--);
+        }
+        else {
+            @dates = (
+                "--since=$since",
+                ($option{until} ? "--until=$option{until}" : ()),
+            );
         }
-    }
 
-    for my $user (keys %users) {
-        my $commits = $users{$user};
-        $users{$user} = {
-            commit_count => scalar keys %{ $users{$user} },
-            $option{commits} ? (commits => [keys %{ $users{$user} }]) : (),
+        for my $branch ($workflow->git->branch(@options)) {
+            next if $branch =~ / -> /;
+            $branch =~ s/^[*]?\s*//;
+            for my $log ( $workflow->git->log( @log, @dates, $branch, ) ) {
+                my ($hash, $name) = split /\s/, $log, 2;
+                $users{$name}{$hash} = 1;
+                $commits++;
+            }
+        }
+
+        for my $user (keys %users) {
+            my $commits = $users{$user};
+            $users{$user} = {
+                commit_count => scalar keys %{ $users{$user} },
+                $option{commits} ? (commits => [keys %{ $users{$user} }]) : (),
+                $option{changes} ? (changes => $self->changes($commits)) : (),
+            };
         }
     }
 
@@ -84,10 +106,66 @@ sub run {
     return;
 }
 
+sub dates {
+    my ($self, $period, $count) = @_;
+    my @dates;
+
+    $period
+        = $period eq 'day'   ? 1
+        : $period eq 'week'  ? 7
+        : $period eq 'month' ? 30
+        : $period eq 'year'  ? 365
+        :                      die "Unknown period '$option{period}' please choose one of day, week, month or year\n";
+
+    my $now = localtime;
+    my $since
+        = $now->wday == 1 ? localtime(time - 3 * $period * 24 * 60 * 60)->ymd
+        : $now->wday == 7 ? localtime(time - 2 * $period * 24 * 60 * 60)->ymd
+        :                   localtime(time - 1 * $period * 24 * 60 * 60)->ymd;
+
+    return @dates;
+}
+
+sub changes {
+    my ($self, $commits) = @_;
+    my %changes = (
+        lines_added   => 0,
+        lines_removed => 0,
+        files         => {},
+        files_added   => 0,
+        files_removed => 0,
+    );
+
+    for my $commit (keys %$commits) {
+        # get the stats from each commit
+        my @show = $workflow->git->show($commit);
+        $changes{lines_added}   += grep {/^[+](?:[^+]|[+][^+]|[+][+]\s|$)/} @show;
+        $changes{lines_removed} += grep {/^[-](?:[^-]|[-][^-]|[-][-]\s|$)/} @show;
+        $changes{files} = {
+            %{ $changes{files} || {} },
+            map {/^[+]{3}\s+b\/(.*)$/; ($1 || "" => 1) }
+            grep {/^[+]{3}\s/}
+            @show
+        };
+        $changes{total}++;
+    }
+    $changes{files} = keys %{ $changes{files} || {} };
+
+    return \%changes;
+}
+
 sub fmt_table {
     my ($self, $users, $total) = @_;
+    my $fmt = "%-25s % 7d";
 
-    print map {sprintf "% 5d $_\n", $users->{$_}{commit_count}}
+    if ($option{changes}) {
+        $fmt .= " % 9d % 9d % 5d";
+        my $fmt2 = $fmt;
+        $fmt2 =~ s/d/s/g;
+        printf "$fmt2\n", qw/Name Commits Added Removed Files/;
+    }
+
+    print map {sprintf "$fmt\n", $_, $users->{$_}{commit_count}, $users->{$_}{changes}{lines_added}, $users->{$_}{changes}{lines_removed}, $users->{$_}{changes}{files}}
         reverse sort {$users->{$a}{commit_count} <=> $users->{$b}{commit_count}}
         grep { $users->{$_}{commit_count} >= ($option{min} || 0) }
         keys %$users;
@@ -121,7 +199,7 @@ git-committers - Stats on the number of commits by committer
 
 =head1 VERSION
 
-This documentation refers to git-committers version 0.96009
+This documentation refers to git-committers version 0.96011
 
 =head1 SYNOPSIS
 
@@ -130,8 +208,10 @@ This documentation refers to git-committers version 0.96009
  OPTIONS:
   -r --remote   Committers to remote branches
   -a --all      Committers to any branch (remote or local)
-  -d --date=YYYY-MM-DD
-                Commits since this date
+  -s --since[=]YYYY-MM-DD
+                Only commits since this date
+  -u --until[=]YYYY-MM-DD
+                Only commits up until this date
   -f --format[=](table|json|csv)
                 Change how the data is presented
                    - table : shows the data in a simple table
@@ -139,7 +219,7 @@ This documentation refers to git-committers version 0.96009
                    - perl  : Dump the data structure
                    - csv   : Like table but as a csv
   -p --period=[day|week|month|year]
-                If --date is not specified this works out the date for the
+                If --since is not specified this works out the date for the
                 last day/week/month/year
   -m --merges   Count merge commits
      --no-merges
@@ -161,6 +241,14 @@ to the git repository.
 
 Executes the git workflow command
 
+=head2 C<dates ($period, $count)>
+
+Returns the C<--since> and C<--until> dates for the C<$period> specified
+
+=head2 C<changes ($commits)>
+
+Calculates the changes for C<$commits>.
+
 =head2 C<fmt_table ()>
 
 Output a table
